#include "Garmin.h"
#include "MacInterface.h"

#define TIMEOUT 5	/* return after this number of seconds without receiving msg */

BYTE gGarminMessage[MAX_LENGTH];

/* --------------------------------------------------------------------------------- */

void sendGPSMessage(BYTE *message, short bytes)
{
	extern short gOutRefNum;
	short i;
	long n;
	BYTE csum = 0;
	BYTE *p = gGarminMessage;
	
	*p++ = 0x10;    /* Start of message */
	for (i=0; i<bytes; i++) {
		*p++ = message[i];
		csum -= message[i];
		if (message[i] == 0x10)
			*p++ = 0x10;
	}
	*p++ = csum;
	if (csum == 0x10)
		*p++ = 0x10;
	*p++ = 0x10;
	*p++ = 0x03;
	n = (long)(p - gGarminMessage);
	FSWrite(gOutRefNum, &n, gGarminMessage);
}

/*
	getGPSMessage() returns the number of bytes in the message. If the
	timeout period has lapsed without completing a message, 0 is returned.
*/

/* --------------------------------------------------------------------------------- */

short getGPSMessage()
{
	extern short gInRefNum;
	extern enum PROTOCOL mode;

	long count = 1L;
	BYTE c, *p = gGarminMessage, last = 0;
	
	short length = 0;
	
	long time = TickCount();
	
	for (;;) {
		if (TickCount() > time + TIMEOUT * 60L) {
			Error("The GPS receiver is not responding.");
			mode = NONE;
			return 0;
		}
			
		while (serialCharsAvail()) {
			if (length >= MAX_LENGTH-1) {
				Error("GPS receiver communication protocol error.");
				mode = NONE;
				return 0;
			}
		
			count = 1L;
			FSRead(gInRefNum, &count, &c);
			
			if (c == 0x10 && last == 0x10)
				last = 0;
			else {
				last = *p++ = c;
				++length;
			}

			if (*(p-1) == 0x03 && *(p-2) == 0x10)
				return length;
		}
	}
}
