#include "Garmin.h"

short gInRefNum=0, gOutRefNum=0;

static const short SERBUFSIZ = 10000;
static Ptr buf;

/* --------------------------------------------------------------------------------- */

OSErr serialOpen(enum PROTOCOL p)
{
	OSErr		err;
	SerShk flags;
	short setup;
	
	switch (p) {
		case GARMIN:
			setup = baud9600 + data8 + stop10 + noParity;
			break;
		
		case NMEA:
			setup = baud4800 + data8 + stop10 + noParity;
			break;
		
		default:
			return 9999;
	}
	
	/* Open Serial Drivers */

	if ((err = OpenDriver(INDRIVER, &gInRefNum)) != noErr)
		return err;
	if ((err = OpenDriver(OUTDRIVER, &gOutRefNum)) != noErr)
		return err;

	/* Reset both input and output, and assign communication protocols */
	
	if ((err = SerReset(gInRefNum, setup)) != noErr)
		return err;
	if ((err = SerReset(gOutRefNum, setup)) != noErr)
		return err;

	/* Disable CTS and XOn/XOff handshaking, events */

	flags.fXOn = 0;
	flags.fCTS = 0;
	flags.errs = 0;
	flags.evts = 0;
	flags.fInX = 0;
	
	if ((err = SerHShake(gInRefNum, &flags)) != noErr)
		return err;
	if ((err = SerHShake(gOutRefNum, &flags)) != noErr)
		return err;
		
	/* Give the serial input driver a SERBUFSIZ buffer */
	if(!(buf = NewPtr(SERBUFSIZ)))
		return MemError();
	if ((err = SerSetBuf(gInRefNum, buf, SERBUFSIZ)) != noErr)
		return err;

	return noErr;
}

/* --------------------------------------------------------------------------------- */

long serialCharsAvail()
{
	long count;

	SerGetBuf(gInRefNum, &count);
	return count;
}

/* --------------------------------------------------------------------------------- */

void serialClose()
{
	if(gInRefNum)
		CloseDriver(gInRefNum);
	if(gOutRefNum)
		CloseDriver(gOutRefNum);
	if (buf)
		DisposPtr(buf);
}