#include "Garmin.h"
#include "MacInterface.h"
#include "Prefs.h"

extern BYTE gGarminMessage[MAX_LENGTH];

static short records(short type);
static short doWaypoint(short type);
static short doTrack(void);
static short doRoute(void);
static short doAlmanac(void);
static short sendRecords(short);
static void copyNumber(BYTE *p, long n);
static char *field(char *a, short n);
static void cpystr(BYTE *p, char *q, short n);
static short getFileData(char *);
static void cleanupDMS(char *s);

static char fileData[MAX_LINE];
static BYTE newTrack;
static BYTE message[100];
static short FileRefNum;

struct PREFS gFilePrefs;

/* --------------------------------------------------------------------------------- */

void sendGPSInfo(short refNum, short type)
{
	extern char gMessageStr[];
	
	short recs, processed = 0;
	short err, result=1;
	char *rType;
	BYTE *term;
	
	if ((err = serialOpen(GARMIN)) != noErr) {
		sprintf(gMessageStr, "The serial port initialization has failed [%d].", err);
		Error(gMessageStr);
		ExitToShell();
	}

	FileRefNum = refNum;
	recs = records(type);
	if (sendRecords(recs)) {	// continue only if a response was received from GPS receiver
		SetFPos(refNum, 1, 0L);
		GetLine(FileRefNum, fileData, 1);
		if (getFileData(fileData)) {
			InitBarGraph();
			switch (type) {
		
				default:
				case ALMANAC:
					rType = "almanac";
					term = almt;
					while (GetLine(FileRefNum, fileData, 0) && processed < recs && result) {
						if (fileData[0] == 'A') {
							result = doAlmanac();
							SetBarGraph((double)(++processed)/(double)recs);
						}
					}
					break;
		
				case ROUTE:
					rType = "route/waypoint";
					term = rtet;
					while (GetLine(FileRefNum, fileData, 0) && processed < recs && result) {
						if (fileData[0] == 'R') {
							result = doRoute();
							++processed;
						}
						if (fileData[0] == 'W') {
							result = doWaypoint(ROUTE);
							++processed;
						}
						SetBarGraph((double)(processed)/(double)recs);
					}
					break;
		
				case TRACK:
					rType = "track";
					term = trkt;
					newTrack = 1;
					while (GetLine(FileRefNum, fileData, 0) && processed < recs && result) {
						if (fileData[0] == 'T') {
							result = doTrack();
							SetBarGraph((double)(++processed)/(double)recs);
						}
						else
							newTrack = 1;
					}
					break;
		
				case WAYPOINT:
					rType = "waypoint";
					term = wptt;
					while (GetLine(FileRefNum, fileData, 0) && processed < recs && result) {
						if (fileData[0] == 'W') {
							result = doWaypoint(WAYPOINT);
							SetBarGraph((double)(++processed)/(double)recs);
						}
					}
					break;
			}
			CloseBarGraph();
			if (result) {
				sendGPSMessage(term, 4);	// send terminator message
				getGPSMessage();	// wait for final response, ignore it
				sprintf(gMessageStr, "%d %s packets were transferred to the GPS receiver.", recs, rType);
				Message(gMessageStr);
			}
			serialClose();
		}
		else
			Error("The file header format is incorrect.");
	}
}

/* --------------------------------------------------------------------------------- */

/*
Format: DDD  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
W    HOME    123 SNOWMASS PL   07/28/1994 20:12:54  40.1777471  -105.0892654
0     1        2                  3                     4            5
TYPE  NAME     COMMENT            DATE                  LATITUDE     LONGITUDE

Format: DMM  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
W    HOME    123 SNOWMASS PL   07/28/1994 20:12:54  4010.665'  -10505.356'
0     1        2                  3                     4            5
TYPE  NAME     COMMENT            DATE                  LATITUDE     LONGITUDE

Format: DMS  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
W    HOME    123 SNOWMASS PL   07/28/1994 20:12:54  4010'39.9" -10505'21.4"
0     1        2                  3                     4            5
TYPE  NAME     COMMENT            DATE                  LATITUDE     LONGITUDE

Format: UTM  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
W    HOME    123 SNOWMASS PL   07/28/1994 20:12:54  13  T  0492400  4447279
0     1        2                  3                     4    5   6         7
TYPE  NAME     COMMENT            DATE                  ZE   ZN  EASTING   NORTHING

Format: BNG  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
W    EXAMPL  BRITISH GRID      07/28/1994 20:12:54  SE  12345    67890
0     1        2                  3                     4    5         6
TYPE  NAME     COMMENT            DATE                  ZONE EASTING   NORTHING

Format: ITM  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
W    EXAMPL  IRISH GRID        07/28/1994 20:12:54  IN  12345    67890
0     1        2                  3                     4    5         6
TYPE  NAME     COMMENT            DATE                  ZONE EASTING   NORTHING

Format: SUI  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
W    EXAMPL  SWISS GRID        07/28/1994 20:12:54  012345    067890
0     1        2                  3                     5          6
TYPE  NAME     COMMENT            DATE                  EASTING    NORTHING

*/

// Returns a result code--zero if GPS receiver did not respond

static short doWaypoint(short type)
{
	double latitude, longitude, x, y;
	long lat, lon, dat;
	short zone;
	char bgzone[5];
	char date[20], nsZone;
	enum {TYPE=0, NAME, COMMENT, DATE, LAT,     LON};
	enum {                             ZE=4,    ZN,     EASTING, NORTHING}; // UTM
	enum {                             BGZ=4,   BGEAST, BGNORTH}; // British, Irish
	enum {                             SEAST=4, SNORTH};  // Swiss
	
	strncpy(date, field(fileData, DATE), 19);
	date[19] = 0;
	
	switch(gFilePrefs.format) {
		case DMS:
			cleanupDMS(fileData);
			latitude = DMStoDegrees(field(fileData, LAT));
			longitude = DMStoDegrees(field(fileData, LON));
			break;
			
		case DMM:
			cleanupDMS(fileData);
			latitude = DMtoDegrees(field(fileData, LAT));
			longitude = DMtoDegrees(field(fileData, LON));
			break;
			
		case DDD:
			sscanf(field(fileData, LAT), "%lf %lf", &latitude, &longitude);
			break;
			
		case UTM:		
			sscanf(field(fileData, ZE), "%d", &zone);
			sscanf(field(fileData, ZN), "%c", &nsZone);
			sscanf(field(fileData, EASTING), "%lf %lf", &x, &y);
			UTMtoDeg(zone, nsZone <= 'M', x, y, &latitude, &longitude);
			break;

		case BNG:
			sscanf(field(fileData, BGZ), "%s", bgzone);
			sscanf(field(fileData, BGEAST), "%lf %lf", &x, &y);
			BNGtoDeg(bgzone, x, y, &latitude, &longitude);
			break;
			
		case ITM:
			sscanf(field(fileData, BGZ), "%s", bgzone);
			sscanf(field(fileData, BGEAST), "%lf %lf", &x, &y);
			ITMtoDeg(bgzone, x, y, &latitude, &longitude);
			break;
			
		case SUI:
			break;
			
		default:
			break;
	}

	translate(0, &latitude, &longitude, gFilePrefs.datum);

	lat = deg2int(latitude);
	lon = deg2int(longitude);
	dat = dt2secs(date, gFilePrefs.offset);

	if (type == WAYPOINT)
		message[0] = '\x23';
	else /* type == ROUTE */
		message[0] = '\x1e';
	message[1] = 58;	/* number of message bytes to follow */

	cpystr(message+2, field(fileData, NAME), 6);
	copyNumber(message+8, lat);
	copyNumber(message+12, lon);
	copyNumber(message+16, dat);
	fileData[strlen(fileData)-1] = 0;		/* replace newline with null */
	cpystr(message+20, field(fileData, COMMENT), 40);

	sendGPSMessage(message, 60);
	return getGPSMessage();	// make sure GPS receiver responded
}

/* --------------------------------------------------------------------------------- */

/* Track file format ( represents a tab character)

T   08/29/1994 13:32:01 40.1446967 -105.1245464 13  T 489391 4443614
0    1                    2           3             4    5  6       7
TYPE DATE                 LAT         LON           ZONE ZN EASTING NORTHING

Format: DDD  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
T    09/09/1994 20:03:49  40.1780207  -105.0901666
0     1                     2            3
TYPE  DATE/TIME             LATITUDE     LONGITUDE

Format: DMM  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
T    09/09/1994 20:03:49  4010.681'  -10505.410'
0     1                     2            3
TYPE  DATE/TIME             LATITUDE     LONGITUDE

Format: DMS  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
T    09/09/1994 20:03:49  4010'40.9" -10505'24.6"
0     1                     2            3
TYPE  DATE/TIME             LATITUDE     LONGITUDE

Format: UTM  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
T    09/09/1994 20:03:49  13  T   0492323   4447310
0     1	                    2    3    4          5
TYPE  DATE/TIME             ZE   ZN   EASTING    NORTHING

Format: BNG  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
T    09/09/1994 20:03:49  SE   12345   67890
0     1	                    2     3        4          5
TYPE  DATE/TIME             ZONE  EASTING  NORTHING

Format: ITM  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
T    09/09/1994 20:03:49  IN   12345   67890
0     1	                    2     3        4          5
TYPE  DATE/TIME             ZONE  EASTING  NORTHING

Format: SUI  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
T    09/09/1994 20:03:49  012345   067890
0     1	                    3         4
TYPE  DATE/TIME             EASTING   NORTHING


Note: Even though the date/time is included in the protocol, the
GPS 45 appears to ignore this field. The only track data that is time
stamped is that actually generated by the GPS 45; actual track data
uploaded from the GPS 45 has correct time stamps.

*/

// Returns a result code--zero if GPS receiver did not respond

static short doTrack()
{
	double latitude, longitude, x, y;
	char date[20], nsZone, bgzone[3];
	long lat, lon, dat;
	short zone;
	enum {TYPE, DATE, LAT, LON};
	enum {            ZE=2,    ZN,     EASTING, NORTHING}; // UTM
	enum {            BGZ=2,   BGEAST, BGNORTH};           // British, Irish
	enum {            SEAST=2, SNORTH};                    // Swiss
	
	strncpy(date, field(fileData, DATE), 19);
	date[19] = 0;
	
	switch(gFilePrefs.format) {
		case DMS:
			cleanupDMS(fileData);
			latitude = DMStoDegrees(field(fileData, LAT));
			longitude = DMStoDegrees(field(fileData, LON));
			break;
			
		case DMM:
			cleanupDMS(fileData);
			latitude = DMtoDegrees(field(fileData, LAT));
			longitude = DMtoDegrees(field(fileData, LON));
			break;
			
		case DDD:
			sscanf(field(fileData, LAT), "%lf %lf", &latitude, &longitude);
			break;
			
		case UTM:
			sscanf(field(fileData, ZE), "%d", &zone);
			sscanf(field(fileData, ZN), "%c", &nsZone);
			sscanf(field(fileData, EASTING), "%lf %lf", &x, &y);
			UTMtoDeg(zone, nsZone <= 'M', x, y, &latitude, &longitude);
			break;

		case BNG:
			sscanf(field(fileData, BGZ), "%s", bgzone);
			sscanf(field(fileData, BGEAST), "%lf %lf", &x, &y);
			BNGtoDeg(bgzone, x, y, &latitude, &longitude);
			break;
			
		case ITM:
			sscanf(field(fileData, BGZ), "%s", bgzone);
			sscanf(field(fileData, BGEAST), "%lf %lf", &x, &y);
			ITMtoDeg(bgzone, x, y, &latitude, &longitude);
			break;
			
		case SUI:
			break;
			
		default:
			break;
	}

	translate(0, &latitude, &longitude, gFilePrefs.datum);

	lat = deg2int(latitude);
	lon = deg2int(longitude);
	dat = dt2secs(date, gFilePrefs.offset);

	message[0] = '\x22';
	message[1] = 13;	/* number of message bytes to follow */
	copyNumber(message+2, lat);
	copyNumber(message+6, lon);
	copyNumber(message+10, dat);
	message[14] = newTrack;		/* new track flag */
	newTrack = 0;
	sendGPSMessage(message, 15);
	return getGPSMessage();	// make sure GPS receiver responded
}

/* --------------------------------------------------------------------------------- */

/* Route file record format ( represents a tab character)
R	1	BOGUS 1
0   1   2
*/

// Returns a result code--zero if GPS receiver did not respond

static short doRoute()
{
	short route;
	
	message[0] = '\x1d';
	message[1] = 21;
	sscanf(field(fileData, 1), "%d", &route);
	message[2] = route;
	cpystr(message+3, field(fileData, 2), 20);
	sendGPSMessage(message, 23);
	return getGPSMessage();	// make sure GPS receiver responded
}

/* --------------------------------------------------------------------------------- */

#define ALM_BYTES 42

// Returns a result code--zero if GPS receiver did not respond

static short doAlmanac()
{
	short i;
	unsigned short x;
	
	message[0] = '\x1f';
	message[1] = ALM_BYTES;
	for (i = 0; i < ALM_BYTES; i++) {
		sscanf(fileData+2+3*i, "%02X", &x);
		message[i+2] = (BYTE)x;
	}
	sendGPSMessage(message, ALM_BYTES+2);
	return getGPSMessage();	// make sure GPS receiver responded
}

/* --------------------------------------------------------------------------------- */

static short records(short type)
{
	short n = 0;
	
	GetLine(FileRefNum, fileData, 1);
	while (GetLine(FileRefNum, fileData, 0)) {
		switch (type) {
			case ALMANAC:
			default:
				if (fileData[0] == 'A')
					++n;
				break;
				
			case ROUTE:
				if (fileData[0] == 'R' || fileData[0] == 'W')
					++n;
				break;
				
			case TRACK:
				if (fileData[0] == 'T')
					++n;
				break;
				
			case WAYPOINT:
				if (fileData[0] == 'W')
					++n;
				break;
		}
	}
	return n;
}

/* --------------------------------------------------------------------------------- */

// Sends the message containing the number of records to follow
// Returns a result code--zero if GPS receiver did not respond

static short sendRecords(short recs)
{
	message[0] = '\x1b';
	message[1] = '\x02';
	message[2] = recs % 256; /* LSB of number of records */
	message[3] = recs / 256; /* MSB */
	sendGPSMessage(message, 4);
	return getGPSMessage();	// make sure GPS receiver responded
}

/* --------------------------------------------------------------------------------- */

static void copyNumber(BYTE *p, long n)
{	
	BYTE *q = (BYTE *)&n;
	
	*p++ = *(q+3);
	*p++ = *(q+2);
	*p++ = *(q+1);
	*p = *q;
}

/* --------------------------------------------------------------------------------- */

/*
	field() returns a pointer to the nth field of a tab-delimited record
	which starts at address a.
*/

static char *field(char *a, short n)
{
	short i = 0;
	while ((i++) < n)
		while (*(++a) != '\t')
			;
	return a+1;
}

/* --------------------------------------------------------------------------------- */

static void cpystr(BYTE *p, char *q, short n)
{
	while (*q != '\t' && *q != '\n' && *q != '\r' && *q != 0) {
		*p++ = *q++;
		--n;
	}
	while (n-- > 0)
		*p++ = ' ';
}

/* --------------------------------------------------------------------------------- */

/* 
   The following function retrieves the file format info from the first
   line of the data file. This is highly dependent upon the following
   format for the first line in the file -- note that there are no
   embedded tabs. See the file "getGPSInfo.c" for further information.

Format: DDD  UTC Offset:  -6.00 hrs  Datum[061]: NAD27 CONUS
        |||              ||||||            |||
0         1         2         3         4         5         6
0123456789012345678901234567890123456789012345678901234567890123456789
*/

static short getFileData(char *data)
{
	extern short nDatums;
	char fmt[10];

	sscanf(data, "%7s", fmt);
	if (strcmp(fmt, "Format:") != 0)
		return 0;
	
	sscanf(data+8, "%3s", fmt);
	if (strcmp(fmt, "DMS") == 0)
		gFilePrefs.format = DMS;
	else if (strcmp(fmt, "DMM") == 0)
		gFilePrefs.format = DMM;
	else if (strcmp(fmt, "DDD") == 0)
		gFilePrefs.format = DDD;
	else if (strcmp(fmt, "UTM") == 0)
		gFilePrefs.format = UTM;
	else if (strcmp(fmt, "BNG") == 0)
		gFilePrefs.format = BNG;
	else if (strcmp(fmt, "ITM") == 0)
		gFilePrefs.format = ITM;
	else if (strcmp(fmt, "SUI") == 0)
		gFilePrefs.format = SUI;
	else return 0;
	
	sscanf(data+25, "%lf", &gFilePrefs.offset);
	if (gFilePrefs.offset < -24.0 || gFilePrefs.offset > 24.0)
		return 0;
		
	sscanf(data+43, "%3d", &gFilePrefs.datum);
	if (gFilePrefs.datum < 0 || gFilePrefs.datum > nDatums-1)
		return 0;
		
	return 1;
}

/* --------------------------------------------------------------------------------- */

// Change degree, minute and second symbols to spaces

static void cleanupDMS(char *s)
{
	while (*++s)
		if (*s == '' || *s == '\'' || *s == '\"')
			*s = ' ';
}
