#include "Garmin.h"
#include "Prefs.h"

#define START 2447892L /* Julian date for 00:00 12/31/89 */

static long date2days(short mm, short dd, short yy);
static void days2date(long julian, short *mm, short *dd, short *yy);

/* dt is the date and time in the format "mm/dd/yyyy hh:mm:ss" and secs
   is the number of seconds since 12/31/1989 00:00:00, as defined by the
   #define START
*/

/* --------------------------------------------------------------------------------- */

char *secs2dt(long secs, short offset)
{
	static char dt[30];
	short mm, dd, yy, h, m, s;
	long days = (secs + (long)(offset*3600.0)) / (24L * 3600L);
	long rem = secs + (long)(offset*3600.0) - days * 24L * 3600L;
	
	days2date(days, &mm, &dd, &yy);
	
	h = rem / 3600L;
	m = (rem - (long)h * 3600L) / 60L;
	s = (rem - (long)h * 3600L) - (long)m * 60L;
	sprintf(dt, "%02d/%02d/%04d %02d:%02d:%02d", mm, dd, yy, h, m, s);

	return dt;
}

/* --------------------------------------------------------------------------------- */

long dt2secs(char *dt, short offset)
{
	short mm, dd, yy, h, m, s;
	
	sscanf(dt, "%d/%d/%d %d:%d:%d", &mm, &dd, &yy, &h, &m, &s);
	return 24L * 3600L * date2days(mm, dd, yy)
			+ 3600L * h + 60L * m + s - (long)(offset*3600.0);
}

/* --------------------------------------------------------------------------------- */

static long date2days(short mm, short dd, short yy)
{
	long jul;
	short ja, jy, jm;
	
	if (yy < 0)
		++yy;
	if (mm > 2) {
		jy = yy;
		jm = mm + 1;
	}
	else {
		jy = yy - 1;
		jm = mm + 13;
	}
	ja = .01 * jy;
	jul = (long)(floor(365.25 * jy) + floor(30.6001 * jm) +
				dd + 1720997 - ja + (long)(0.25 * ja));

	return jul-START;
}

/* --------------------------------------------------------------------------------- */

static void days2date(long days, short *mm, short *dd, short *yy)
{
	long ja, jalpha, jb, jc, jd, je;

	jalpha = ((float)(days+START-1867216)-0.25)/36524.25;
	ja = days+START+1+jalpha-(long)(0.25*jalpha);

	jb = ja + 1524L;
	jc = 6680.0 + ((double)(jb - 2439870) - 122.1) / 365.25;
	jd = 365.25 * jc;
	je = (jb - jd) / 30.6001;
	*dd = jb - jd - (short)(30.6001 * je);
	*mm = je - 1;
	if (*mm > 12)
		*mm -= 12;
	*yy = jc - 4715;
	if (*mm > 2)
		--(*yy);
	if (*yy <= 0)
		--(*yy);
}

