#include "Garmin.h"

static const char *zoneID0 = "STUQRNOPLMHJKFGCDEABXYZVW";
static const char *zoneID1 = "VWXYZQRSTULMNOPFGHJKABCDE";

// The k0 constant was empirically determined to give the best fit to the data
// from the GPS 45 and the PCX5 program. It should be 0.9996, but this number
// resulted in slight errors in the data.

static const double lat0 = 49.0;	// reference transverse mercator latitude
static const double lon0 = -2.0;	//   and longitude
static const double k0 = 0.99960133;

/* --------------------------------------------------------------------------------- */

void DegToBNG(double lat, double lon, char *zone, double *x, double *y)
{
	short X0, Y0, X1, Y1;

	strcpy(zone, "--");
	
	toTM(lat, lon, lat0, lon0, k0, x, y);
	
	*x = floor(*x + 400000.0 + 0.5);		// add false easting and northing
	*y = floor(*y - 100000.0 + 0.5);		//   and round to nearest meter

	if (*x < 0.0 || *x >1.0e6 || *y < 0.0 || *y > 2.5e6) {	// Invalid range
		*x = 0.0;		// return with zone = "--" and set x and y to 0.0
		*y = 0.0;
		return;
	}
	
	X0 = (long)*x / 500000;
	Y0 = (long)*y / 500000;

	X1 = ((long)*x / 100000) % 5;
	Y1 = ((long)*y / 100000) % 5;
	
	*x = (long)*x % 100000;
	*y = (long)*y % 100000;

	zone[0] = zoneID0[5 * Y0 + X0];
	zone[1] = zoneID1[5 * Y1 + X1];
}

/* --------------------------------------------------------------------------------- */

void BNGtoDeg(char *zone, double x, double y, double *lat, double *lon)
{
	int m, n;
	
	if (strcmp(zone, "--") == 0 || x < 0.0 || x > 100000.0 || y < 0.0 || y > 100000.0) {
		*lat = 0.0;
		*lon = 0.0;
		return;
	}
	
	m = strchr(zoneID0, zone[0]) - zoneID0;
	n = strchr(zoneID1, zone[1]) - zoneID1;
	
	x += (m % 5) * 500000.0 + (n % 5) * 100000.0 - 400000.0;
	y += (m / 5) * 500000.0 + (n / 5) * 100000.0 + 100000.0;
	
	fromTM(x, y, lat0, lon0, k0, lat, lon);
}


