#include "Garmin.h"

static const char *zoneID = "VWXYZQRSTULMNOPFGHJKABCDE";

// The reference latitude and k0 factor were determined empirically to fit
// the GPS data

static const double lat0 = 53.4999886;	// reference transverse mercator latitude
static const double lon0 = -8.0;		//   and longitude
static const double k0 = 1.000035;

/* --------------------------------------------------------------------------------- */

void DegToITM(double lat, double lon, char *zone, double *x, double *y)
{
	short X, Y;

	strcpy(zone, "--");
	
	toTM(lat, lon, lat0, lon0, k0, x, y);
	
	*x = floor(*x + 200000.0 + 0.5);		// add false easting and northing
	*y = floor(*y + 250000.0 + 0.5);		//   and round to nearest meter

	if (*x < 0.0 || *x >500000.0 || *y < 0.0 || *y > 500000.0) {	// Invalid range
		*x = 0.0;		// return with zone = "--" and set x and y to 0.0
		*y = 0.0;
		return;
	}

	X = (long)*x / 100000;
	Y = (long)*y / 100000;

	*x = (long)*x % 100000;
	*y = (long)*y % 100000;
	
	zone[0] = 'I';
	zone[1] = zoneID[5 * Y + X];
}

/* --------------------------------------------------------------------------------- */

void ITMtoDeg(char *zone, double x, double y, double *lat, double *lon)
{
	int n;
	
	if (strcmp(zone, "--") == 0 || x < 0.0 || x > 100000.0 || y < 0.0 || y > 100000.0) {
		*lat = 0.0;
		*lon = 0.0;
		return;
	}

	n = strchr(zoneID, zone[1]) - zoneID;
	
	x += (n % 5) * 100000.0 - 200000.0;
	y += (n / 5) * 100000.0 - 250000.0;
	
	fromTM(x, y, lat0, lon0, k0, lat, lon);
}

