#include "MacInterface.h"
#include "Garmin.h"
#include "NMEA.h"

enum PROTOCOL mode = NONE;

static void AdjustMenus(void);
static void HandleAppleChoice(short theItem);
static void HandleEditChoice(short theItem);
static void HandleFileChoice(short theItem);
static void HandleMenuChoice(long menuChoice);
static void HandleMouseDown(void);
static void HandleUploadChoice(short theItem);
static void HandleDownloadChoice(short theItem);
static void HandleCommChoice(short theItem);
static char *defaultFName(short n);
static void doDialogEvent(EventRecord theEvent);

static short IsDAWindow(WindowPtr whichWindow);
static EventRecord gTheEvent;

/* --------------------------------------------------------------------------------- */

void HandleEvent()
{
	extern Boolean gWNEImplemented;
	
	char theChar;
	Boolean evtOccurred;
	
	if (gWNEImplemented)
		evtOccurred = WaitNextEvent(everyEvent, &gTheEvent, MIN_SLEEP, NIL_MOUSE_REGION);
	else {
		SystemTask();
		evtOccurred = GetNextEvent(everyEvent, &gTheEvent);
	}
	
	if (evtOccurred) {
		if (IsDialogEvent(&gTheEvent))
			doDialogEvent(gTheEvent);
		else {
			switch(gTheEvent.what) {
				case mouseDown:
					HandleMouseDown();
					break;
				
				case keyDown:
				case autoKey:
					theChar = gTheEvent.message & charCodeMask;
					if ((gTheEvent.modifiers & cmdKey) != 0) {
						AdjustMenus();
						HandleMenuChoice(MenuKey(theChar));
					}
					break;
		
				case updateEvt:
					break;
			}
		}
	}
}

/* --------------------------------------------------------------------------------- */

static void HandleMouseDown()
{
	WindowPtr whichWindow;
	short thePart;
	long menuChoice;
	Rect dragRect;

	extern short gDoingNMEA;
	
	thePart = FindWindow(gTheEvent.where, &whichWindow);
	switch (thePart) {
	
		case inMenuBar:
			AdjustMenus();
			menuChoice = MenuSelect(gTheEvent.where);
			HandleMenuChoice(menuChoice);
			break;
			
		case inSysWindow:
			SystemClick(&gTheEvent, whichWindow);
			break;
			
		case inDrag:
			dragRect = qd.screenBits.bounds;
			dragRect.left += 30;
			dragRect.right -= 30;
			dragRect.bottom -= 30;
			DragWindow(whichWindow, gTheEvent.where, &dragRect);
			break;
				
		case inContent:
			/* SelectWindow(whichWindow); */
			break;
		
		case inGoAway:
			if (TrackGoAway(whichWindow,gTheEvent.where))
				gDoingNMEA = 0;
			break;
	}
}

/* --------------------------------------------------------------------------------- */

static void AdjustMenus()
{
	extern MenuHandle gEditMenu;
	
	if (IsDAWindow(FrontWindow())) {
		EnableItem(gEditMenu, UNDO_ITEM);
		EnableItem(gEditMenu, CUT_ITEM);
		EnableItem(gEditMenu, COPY_ITEM);
		EnableItem(gEditMenu, PASTE_ITEM);
		EnableItem(gEditMenu, CLEAR_ITEM);
	}
	else {
		DisableItem(gEditMenu, UNDO_ITEM);
		DisableItem(gEditMenu, CUT_ITEM);
		DisableItem(gEditMenu, COPY_ITEM);
		DisableItem(gEditMenu, PASTE_ITEM);
		DisableItem(gEditMenu, CLEAR_ITEM);
	}
}

/* --------------------------------------------------------------------------------- */

static short IsDAWindow(WindowPtr whichWindow)
{
	if (whichWindow == 0L)
		return(FALSE);
	else /* DA windows have negative windowKinds */
		return (((WindowPeek)whichWindow)->windowKind < 0);
}

/* --------------------------------------------------------------------------------- */

static void HandleMenuChoice(long menuChoice)
{
	short theMenu, theItem;
	
	if (menuChoice) {
	
		theMenu = HiWord(menuChoice);
		theItem = LoWord(menuChoice);
		
		switch(theMenu) {
			case APPLE_MENU_ID:
				HandleAppleChoice(theItem);
				break;

			case FILE_MENU_ID:
				HandleFileChoice(theItem);
				break;

			case EDIT_MENU_ID:
				HandleEditChoice(theItem);
				break;

			case COMM_MENU_ID:
				HandleCommChoice(theItem);
				break;
				
			case UPLOAD_MENU_ID:
				HandleUploadChoice(theItem);
				break;
				
			case DOWNLOAD_MENU_ID:
				HandleDownloadChoice(theItem);
				break;
		}
		HiliteMenu(0);
	}
}

/* --------------------------------------------------------------------------------- */

static void HandleAppleChoice(short theItem)
{
	extern MenuHandle gAppleMenu;
	
	Str255 accName;
	short accNumber;

	switch(theItem) {
		case ABOUT_ITEM:
			doAbout();
			break;
		
		default:
			GetItem(gAppleMenu, theItem, accName);
			accNumber = OpenDeskAcc(accName);
			break;
	}
}

/* --------------------------------------------------------------------------------- */

static void HandleFileChoice(short theItem)
{
	extern Boolean gDone;
	
	switch( theItem) {
		
		case PREFS_ITEM:
			ChangePrefs();
			break;
		
		case QUIT_ITEM:
			gDone = TRUE;
			break;
	}
}

/* --------------------------------------------------------------------------------- */

static void HandleEditChoice(short theItem)
{
	SystemEdit(theItem - 1);
}

/* --------------------------------------------------------------------------------- */


static void HandleUploadChoice(short theItem)
{
	short refNum;
	SFReply reply;
	
	if (mode != GARMIN) {	/* Only display the message when the mode changes */
		Message("Set the GPS receiver I/O SETUP to GRMN/GRMN.");
		mode = GARMIN;
	}
	strcpy((char *)reply.fName, defaultFName(theItem));
	CtoPstr((char *)reply.fName);
	
	if ( (refNum = OpenOutFile(reply)) != 0 ) {
		switch (theItem) {
			case ALM_ITEM:
				getGPSInfo(refNum, ALMANAC);
				break;
	
			case RTE_ITEM:
				getGPSInfo(refNum, ROUTE);
				break;
	
			case TRK_ITEM:
				getGPSInfo(refNum, TRACK);
				break;
	
			case WPT_ITEM:
				getGPSInfo(refNum, WAYPOINT);
				break;
		}
	}
	FSClose(refNum);
}

/* --------------------------------------------------------------------------------- */

static void HandleDownloadChoice(short theItem)
{
	short refNum;
	SFReply reply;
	
	if (mode != GARMIN) {	/* Only display the message when the mode changes */
		Message("Set the GPS receiver I/O SETUP to GRMN/GRMN.");
		mode = GARMIN;
	}

	if ( (refNum = OpenInFile(&reply)) != 0 ) {
		switch (theItem) {
			case ALM_ITEM:
				sendGPSInfo(refNum, ALMANAC);
				break;
				
			case RTE_ITEM:
				sendGPSInfo(refNum, ROUTE);
				break;
			
			case TRK_ITEM:
				sendGPSInfo(refNum, TRACK);
				break;
	
			case WPT_ITEM:
				sendGPSInfo(refNum, WAYPOINT);
				break;
					
		}
		FSClose(refNum);
	}
}

/* --------------------------------------------------------------------------------- */

static void HandleCommChoice(short theItem)
{
	short refNum;
	SFReply reply;
	
	if (theItem == NMEA_ITEM) {
		if (mode != NMEA) {	/* Only display the message when the mode changes */
			Message("Set the GPS receiver I/O SETUP to NONE/NMEA (or RTCM/NMEA) and select the NMEA 0183 2.0 protocol.");
			mode = NMEA;
		}

		strcpy((char *)reply.fName, defaultFName(5));
		CtoPstr((char *)reply.fName);
		
		if ( (refNum = OpenOutFile(reply)) != 0 ) {
			doNMEA(refNum);
		}
		FSClose(refNum);
	}
}

/* --------------------------------------------------------------------------------- */

static char *defaultFName(short n)
{
	char *dflt[] = {	
		"GPS 45 Almanac Data",
		"GPS 45 Route Data",
		"GPS 45 Track Data",
		"GPS 45 Waypoint Data",
		"NMEA 0183 Data"
	};
	return dflt[n-1];
}

/* --------------------------------------------------------------------------------- */

static void doDialogEvent(EventRecord gTheEvent)
{
	DialogPtr whichDlog;
	short itemHit;
	
	DialogSelect(&gTheEvent, &whichDlog, &itemHit);

	/* no processing required */
}

