#include "Garmin.h"
#include "MacInterface.h"
#include "Prefs.h"

struct PREFS gPrefs;

// callback procedures used from within the dialog manager

static pascal void datumListProc(WindowPtr theDialog, short theItem);
static pascal void ButtonProc(WindowPtr theDialog, short theItem);
static pascal Boolean prefDlgFilter (DialogPtr theDialog, EventRecord *theEvent, short *itemHit);

// Other prototypes

static unsigned char *Datum(short n);
static void doRadioButtons(WindowPtr theDialog);
static void flashButton(DialogPtr theDialog, short theButton);

static ListHandle theList;

/* --------------------------------------------------------------------------------- */

void ChangePrefs()
{
	short	n;					// Offset into datum array
	struct PREFS tempPrefs;
	DialogPtr 	theDialog;  	// the dialog

	// Variables used in GetDItem and SetDItem

	short	type;
	Handle	theHandle;
	Rect	iRect;

	short	itemHit;			// Item hit returned from ModalDialog
	Point	theCell; 			// Used to get which cell in list the user selected
	char	s[DESC_LEN+1];		// String that cell contains

	// temporaty storage of current prefs

	tempPrefs = gPrefs;

	// Get Dialog from resource file

	theDialog = GetNewDialog(DIALOGID, nil, (WindowPtr)-1);

	// Grab the information for the LISTITEM using GetDItem,
	// then using SetDItem, set up a procedure to draw the list.

	GetDItem(theDialog, LISTITEM, &type, &theHandle, &iRect);
	SetDItem(theDialog, LISTITEM, type, (Handle)datumListProc, &iRect);

	// Grab the information for a DUMMYITEM using GetDItem.
	// this will allow a proc to be called, I then will use that proc
	// to draw the ok button as the default.

	GetDItem(theDialog, DUMMYITEM, &type, &theHandle, &iRect);
	SetDItem(theDialog, DUMMYITEM, type, (Handle)ButtonProc, &iRect);

	GetDItem(theDialog, DESCITEM, &type, &theHandle, &iRect);
	SetIText(theHandle, Datum(gPrefs.datum));

	s[0]=4;
	memcpy(s+1, &gPrefs.creator, 4);
	GetDItem(theDialog, CREATITEM, &type, &theHandle, &iRect);
	SetIText(theHandle, (unsigned char *)s);

	sprintf(s, "%.1f", gPrefs.offset);
	CtoPstr(s);
	GetDItem(theDialog, OFFSETITEM, &type, &theHandle, &iRect);
	SetIText(theHandle, (unsigned char *)s);
	SelIText(theDialog, OFFSETITEM, 0, 32767);
	
	doRadioButtons(theDialog);

	// Now show the dialog on the screen.
	// NOTE:  This is when the above procs to draw the list item, and default
	//       button will be called.

	ShowWindow(theDialog);

	// Now enter modal dialog loop

	do {
		ModalDialog(prefDlgFilter, &itemHit);

		// If the ListItem was hit, check to see if any cell was selected.
		// It it was, then update the datum description

		// This is where you would put your code to handle cell selections!

		switch (itemHit) {
			case LISTITEM:
				theCell.h = theCell.v = 0;
				if (LGetSelect(TRUE, &theCell, theList)) {
	
					if ((n = LLastClick(theList).v) >= 0) {
						GetDItem(theDialog, DESCITEM, &type, &theHandle, &iRect);
						SetIText(theHandle, Datum(n));
						gPrefs.datum = n;
					}
				}
				break;
				
			case DMSITEM:
				gPrefs.format = DMS;
				break;
				
			case DMMITEM:
				gPrefs.format = DMM;
				break;
				
			case DDDITEM:
				gPrefs.format = DDD;
				break;
				
			case UTMITEM:
				gPrefs.format = UTM;
				break;
			
			case BNGITEM:
				gPrefs.format = BNG;
				break;
				
			case ITMITEM:
				gPrefs.format = ITM;
				break;
				
			case SUIITEM:
				gPrefs.format = SUI;
				break;
				
			default:
				break;
		}
		doRadioButtons(theDialog);
		ShowWindow(theDialog);
	} while (itemHit != OKITEM && itemHit != CANCELITEM);

	GetDItem(theDialog, CREATITEM, &type, &theHandle, &iRect);
	*(long *)(s+1) = '    ';
	GetIText(theHandle, (unsigned char *)s);
	gPrefs.creator = *(long *)(s+1);

	GetDItem(theDialog, OFFSETITEM, &type, &theHandle, &iRect);
	GetIText(theHandle, (unsigned char *)s);
	PtoCstr((unsigned char *)s);
	sscanf(s, "%lf", &gPrefs.offset);

	if (gPrefs.offset < -24.0 || gPrefs.offset > 24.0) {
		SysBeep(10);
		gPrefs.offset = 0.0;
	}

	if (itemHit == CANCELITEM)
		gPrefs = tempPrefs;
	else
		SavePrefs();

	LDispose(theList);
	DisposDialog(theDialog);
}

/* --------------------------------------------------------------------------------- */

/*
	This is the dialog filter.  I use this so that I can use LClick to
	keep track of clicks in the List Item.
*/

pascal Boolean prefDlgFilter(DialogPtr theDialog, EventRecord *theEvent, short *itemHit)
{
	Rect	iRect;
	short	type;
	Handle	iHndle;
	GrafPtr	savePort;
	Point	p;
	
	switch (theEvent->what) {
		case keyDown:
			// if key was pressed, handle it

			switch ((theEvent->message) & charCodeMask) {
				case 0x0d:	// Return key
				case 0x03:	// Enter key
					*itemHit = OKITEM;
					flashButton(theDialog, OKITEM);
					return TRUE;

				case 0x1b:	// ESC key
					*itemHit = CANCELITEM;
					flashButton(theDialog, CANCELITEM);
					return TRUE;

				case 0x2e:	// period key--check for command key modifier
					if ((theEvent->modifiers) & cmdKey) {
						*itemHit = CANCELITEM;
						flashButton(theDialog, CANCELITEM);
						return TRUE;
					}
					else
						return FALSE;

				default:
					return FALSE;
			}

		case mouseDown:

			// Get where mouse click occured in global coordinates.

			p = theEvent->where;

			//   Save the current port.
			//   Then make sure port is set to the dialog.

			GetPort (&savePort);
			SetPort (theDialog);

			// Convert the coordinates to local to the dialog window

			GlobalToLocal(&p);

			// Since I am only concerned with mouse clicks in the user item,
			// get information for that item.

			GetDItem(theDialog, LISTITEM, &type, &iHndle, &iRect);

			// If the mouse click was not in LISTITEM, then let ModalDialog handle it.

			if (!PtInRect(p, &iRect)) {
				SetPort(savePort);
				return FALSE;
			}

			// Mouse Click was in list item,
			// Set the itemHit to be the LISTITEM,
			// and call LClick on itemhandle (the list).

			*itemHit = LISTITEM;
			LClick(p, theEvent->modifiers, (ListHandle)iHndle);

			// Reset Port, and let ModalDialog know that we handled the event.

			SetPort(savePort);
			return TRUE;

		default :
			return FALSE;
	}
}

/* --------------------------------------------------------------------------------- */

/*
	This is the proc that the I use to draw a border around
	the OK Button to mark it as the default item.
*/

pascal void ButtonProc(WindowPtr theDialog, short theItem)
{
	Rect	iRect;
	Handle	iHndl;
	short	iType;

	//  Grab Information for OKITEM out of the Dialog

	GetDItem(theDialog, OKITEM, &iType, &iHndl, &iRect);

	// Now frame it

	PenSize(3, 3);
	InsetRect(&iRect, -4, -4);
	FrameRoundRect(&iRect, 16, 16);
}

/* --------------------------------------------------------------------------------- */

/*
	This proc is used to create the list for the LISTITEM,
	and then "install" it into the dialog item list
*/

pascal void datumListProc(WindowPtr theDialog, short theItem)
{
	extern struct DATUM const gDatum[];
	extern short nDatums;

	Rect 	iRect, rView, rBounds;
	short 	v;
	char 	s[DESC_LEN+1];
	Point 	pCellSz, theCell;
	Handle	theHandle;
	short 	itype;

	GetDItem(theDialog, DESCITEM, &itype, &theHandle, &iRect);

	// Now frame it

	PenSize(1, 1);
	InsetRect(&iRect, -4, -4);
	FrameRect(&iRect);

	// Grab item information

	GetDItem(theDialog, theItem, &itype, &theHandle, &iRect);

	// Set up view for the list.  Notice that there is some
	// margin left on all sides for the frame, and the verical scroll bar

	rView = iRect;
	rView.right -= 16;
	rView.left +=1;
	rView.bottom -=1;
	rView.top += 1;

	// list array is 1 colume with nDatums rows

	SetRect(&rBounds, 0, 0, 1, nDatums);

	// force auto calculations when displaying the cells.

	pCellSz.h = 0;
	pCellSz.v = 0;

	// create list and draw it.

	theList = LNew (&rView, &rBounds, pCellSz, 0, theDialog, TRUE, TRUE, FALSE, TRUE);

	if (!theList) {
		Message("Unable to allocate list");
		return;
	}

	// Now initialize the cells

	for (v=0; v < nDatums; v++) {
		strcpy (s, gDatum[v].name);
		SetPt(&theCell, 0, v);
		LSetCell(s, strlen(s), theCell, theList);
	}

	// Select the cell corresponding to the selected datum and scroll to it

	SetPt(&theCell, 0, gPrefs.datum);
	LSetSelect(TRUE, theCell, theList);
	LAutoScroll(theList);
	
	// Draw a frame around the user item

	FrameRect (&iRect);

	// associate the new list handle with this user item

	SetDItem(theDialog, theItem, 0, (Handle)theList, &iRect);
}

/* --------------------------------------------------------------------------------- */

/*
	Datum(n) returns a Pascal string with the datum name and the
	ellipsoid parameters from the array gDatum[].
*/

static unsigned char *Datum(short n)
{
	extern struct ELLIPSOID const gEllipsoid[];
	extern struct DATUM const gDatum[];
	static char s[255];

	sprintf(s,
		"Datum: %s\rEllipsoid: %s\r  a = %.3f m\r  f = 1/%.9f\rGeocentric displacement:\r  dx = %d m\r  dy = %d m\r  dz = %d m\r",
		gDatum[n].name,
		gEllipsoid[gDatum[n].ellipsoid].name,
		gEllipsoid[gDatum[n].ellipsoid].a,
		gEllipsoid[gDatum[n].ellipsoid].invf,
		gDatum[n].dx,
		gDatum[n].dy,
		gDatum[n].dz);
	CtoPstr(s);
	return (unsigned char *)s;
}

/* --------------------------------------------------------------------------------- */

static void doRadioButtons(WindowPtr theDialog)
{
	Rect	iRect;
	Handle	iHandle;
	short	iType;
	ControlHandle	CItem;
	short	i;
	
	for (i = DMSITEM; i <= SUIITEM; i++) {
		GetDItem(theDialog, i, &iType, &iHandle, &iRect);
		CItem = (ControlHandle)iHandle;
		SetCtlValue (CItem, 0);
	}

	switch (gPrefs.format) {
		case DMS:
			GetDItem(theDialog, DMSITEM, &iType, &iHandle, &iRect);
			CItem = (ControlHandle)iHandle;
			SetCtlValue (CItem, 1);
			break;

		case DMM:
			GetDItem(theDialog, DMMITEM, &iType, &iHandle, &iRect);
			CItem = (ControlHandle)iHandle;
			SetCtlValue (CItem, 1);
			break;

		case DDD:
			GetDItem(theDialog, DDDITEM, &iType, &iHandle, &iRect);
			CItem = (ControlHandle)iHandle;
			SetCtlValue (CItem, 1);
			break;

		case UTM:
			GetDItem(theDialog, UTMITEM, &iType, &iHandle, &iRect);
			CItem = (ControlHandle)iHandle;
			SetCtlValue (CItem, 1);
			break;
		
		case BNG:
			GetDItem(theDialog, BNGITEM, &iType, &iHandle, &iRect);
			CItem = (ControlHandle)iHandle;
			SetCtlValue (CItem, 1);
			break;

		case ITM:
			GetDItem(theDialog, ITMITEM, &iType, &iHandle, &iRect);
			CItem = (ControlHandle)iHandle;
			SetCtlValue (CItem, 1);
			break;

		case SUI:
			GetDItem(theDialog, SUIITEM, &iType, &iHandle, &iRect);
			CItem = (ControlHandle)iHandle;
			SetCtlValue (CItem, 1);
			break;

		default:
			break;
	}
}

/* --------------------------------------------------------------------------------- */

static void flashButton(DialogPtr theDialog, short theButton)
{
	Rect	iRect;
	short	type;
	Handle	iHndle;
	long	waitTicks;
	
	GetDItem(theDialog, theButton, &type, &iHndle, &iRect);
	HiliteControl((ControlHandle)iHndle, inButton);
	Delay(8, &waitTicks);
	HiliteControl((ControlHandle)iHndle, FALSE);
}