#include "Garmin.h"
#include "NMEA.h"

extern struct OUTDATA gNMEAdata;

static char *field(char *sentence, short n);

/* --------------------------------------------------------------------------------- */

/*
	The time field in the GPRMC sentence is in the format hhmmss; the date field
	is in the format ddmmyy. The output will be in the format:
	mm/dd/yyyy hh:mm:ss
	01234567890123456789
*/

void processGPRMC(char *sentence)
{
	char s[20], d[10];
		
	sscanf(field(sentence, 9), "%s", d);	// Date: ddmmyy
	strncpy(s, d+2, 2);						// copy month
	strncpy(s+3, d, 2);						// copy date
	strncpy(s+6, "19", 2);					// 20th century
	strncpy(s+8, d+4, 2);					// copy year
	
	sscanf(field(sentence, 1), "%s", d);	// Time: hhmmss
	strncpy(s+11, d, 2);					// copy hours
	strncpy(s+14, d+2, 2);					// copy minutes
	strncpy(s+17, d+4, 2);					// copy seconds
	
	s[2] = s[5] = '/';					// add the '/'s, ':'s, ' ' and string terminator
	s[10] = ' ';
	s[13] = s[16] = ':';
	s[19] = '\0';
	
	strcpy(gNMEAdata.utc, s);
	
	sscanf(field(sentence, 7), "%lf", &gNMEAdata.speed);
	sscanf(field(sentence, 8), "%lf", &gNMEAdata.track);
}

/* --------------------------------------------------------------------------------- */

void processGPGGA(char *sentence)
{
	double lat, lon, d, m;
	char str[20];

	sscanf(field(sentence, 2), "%lf", &lat);
	sscanf(field(sentence, 4), "%lf", &lon);
	sscanf(field(sentence, 7), "%d",  &gNMEAdata.satellites);
	sscanf(field(sentence, 9), "%lf", &gNMEAdata.altitude);
	
	m = 100.0 * modf(lat / 100.0, &d);
	lat = d + m / 60.0;
	
	strcpy(str, field(sentence, 3));
	if (str[0] == 'S')
		lat = -lat;
	gNMEAdata.latitude = lat;

	m = 100.0 * modf(lon / 100.0, &d);
	lon = d + m / 60.0;
	
	strcpy(str, field(sentence, 5));
	if (str[0] == 'W')
		lon = -lon;
	gNMEAdata.longitude = lon;
}

/* --------------------------------------------------------------------------------- */

void processGPGSA(char *sentence)
{
	sscanf(field(sentence, 2),  "%d",  &gNMEAdata.mode);
	sscanf(field(sentence, 15), "%lf", &gNMEAdata.pdop);
	sscanf(field(sentence, 16), "%lf", &gNMEAdata.hdop);
	sscanf(field(sentence, 17), "%lf", &gNMEAdata.vdop);
}

/* --------------------------------------------------------------------------------- */

short checksum(char *sentence)
{
	BYTE sum = '\0';
	char c, *p = sentence, csum[3];
	
	while ((c = *p++) != '*')
		sum ^= c;

	sprintf(csum, "%02X", sum);
	return (strncmp(csum, p, 2) == 0);
}

/* --------------------------------------------------------------------------------- */

/* field() returns a string containing the nth comma delimited field from sentence string */

static char *field(char *sentence, short n)
{
	static char result[100];
	char *p = sentence;
	
	while (n-- > 0)
		while (*p++ != ',')
			;
	strcpy(result, p);
	p = result;
	while (*++p != ',')
		;
	*p = '\0';
	return result;
}

